"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs2/helpers/interopRequireDefault");

var _assign = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/assign"));

var _es = _interopRequireDefault(require("aggregation/es6"));

var _eventemitter = _interopRequireDefault(require("eventemitter3"));

var _ipcPubsub = _interopRequireDefault(require("ipc-pubsub"));

var _ipcKeyval = _interopRequireDefault(require("ipc-keyval"));

var _pureUuid = _interopRequireDefault(require("pure-uuid"));

var _gts1Tracking = _interopRequireDefault(require("./gts-1-tracking"));

var _gts2Resolver = _interopRequireDefault(require("./gts-2-resolver"));

var _gts3Evaluation = _interopRequireDefault(require("./gts-3-evaluation"));

/*
**  GraphQL-Tools-Subscribe -- Subscription Framework for GraphQL-Tools
**  Copyright (c) 2016-2019 Dr. Ralf S. Engelschall <rse@engelschall.com>
**
**  Permission is hereby granted, free of charge, to any person obtaining
**  a copy of this software and associated documentation files (the
**  "Software"), to deal in the Software without restriction, including
**  without limitation the rights to use, copy, modify, merge, publish,
**  distribute, sublicense, and/or sell copies of the Software, and to
**  permit persons to whom the Software is furnished to do so, subject to
**  the following conditions:
**
**  The above copyright notice and this permission notice shall be included
**  in all copies or substantial portions of the Software.
**
**  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
**  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
**  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
**  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
**  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
**  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
**  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

/*  external dependencies  */

/*  internal dependencies  */

/*  the API class  */
class GraphQLToolsSubscribe extends (0, _es.default)(_gts1Tracking.default, _gts2Resolver.default, _gts3Evaluation.default) {
  /*  the class constructor  */
  constructor(options = {}) {
    super();
    this.options = (0, _assign.default)({
      pubsub: "spm",
      keyval: "spm"
    }, options);
    this.keyval = new _ipcKeyval.default(this.options.keyval);
    this.pubsub = new _ipcPubsub.default(this.options.pubsub);
    this.subscription = null;
    this.emitter = new _eventemitter.default();
    this.uuid = new _pureUuid.default(1).format();
  }
  /*  provide event listening  */


  on(name, handler) {
    this.emitter.on(name, handler);
    return () => {
      this.emitter.removeListener(name, handler);
    };
  }
  /*  provide event emitting  */


  emit(name, ...args) {
    this.emitter.emit(name, ...args);
    return this;
  }
  /*  open service  */


  async open() {
    await this.keyval.open();
    await this.pubsub.open();
    this.subscription = await this.pubsub.subscribe("outdated", sids => {
      this.__scopeOutdatedEvent(sids);
    });
    return this;
  }
  /*  close service  */


  async close() {
    this.subscription.unsubscribe();
    this.subscription = null;
    await this.keyval.close();
    await this.pubsub.close();
    return this;
  }

}
/*  export the traditional way for interoperability reasons
    (as Babel would export an object with a 'default' field)  */


module.exports = GraphQLToolsSubscribe;
